unit demo;

{$MODE objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, opbitmap, lazbridge,
  ExtCtrls, Buttons, StdCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    chbFromOrig: TCheckBox;
    ComboBox1: TComboBox;
    ComboBox2: TComboBox;
    Image1: TImage;
    Image2: TImage;
    Memo1: TMemo;
    Panel1: TPanel;
    Panel2: TPanel;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure ComboBox1CloseUp(Sender: TObject);
    procedure ComboBox2CloseUp(Sender: TObject);
    procedure FormChangeBounds(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chbFromOrigChange(Sender: TObject);
  private
    { private declarations }
    procedure PaintPalette;
  public
    { public declarations }
  end;

var
  Form1: TForm1;
  OrigBmp, ViewBmp: TCanvasOPBitmap;

implementation

function ByteToString(Bytes: cardinal): string;
var temp: Cardinal;
begin
  if Bytes < $400 then
    result := inttostr(Bytes) + ' Bytes'
  else
    if Bytes < $100000 then
    begin
      result := inttostr((Bytes div $400)) + ' KB';
    end
    else
    begin
      Result := FloatToStr(((Bytes * 100) div $100000) / 100) + ' MB';
    end;
end;

{ TForm1 }


procedure TForm1.ComboBox1CloseUp(Sender: TObject);
begin
  Application.ProcessMessages;
  if chbFromOrig.Checked then ViewBmp.Assign(OrigBmp);
  if ViewBmp.Width > 0 then
  begin
    ViewBmp.Canvas.Resample(Image1.width, Image1.height);
    ViewBmp.ReductionMode := TReductionMode(ComboBox2.ItemIndex);
    ViewBmp.PixelFormat := TPixelFormat(Combobox1.ItemIndex + 1);
    AssignOpBitmapToBitmap(ViewBmp, Image1.Picture.Bitmap);
    Memo1.Text := 'Daten: ' + ByteToString(ViewBmp.GetDataSize);
    if ViewBmp.BPP <= 8 then
    Memo1.Lines.Add('Palettengroesse: ' + Inttostr(ViewBmp.ColorTableSize));
    PaintPalette;
    Image1.Repaint;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  ViewBmp.Monochrome := true;
  AssignOpBitmapToBitmap(ViewBmp, Image1.Picture.Bitmap);
  Image1.Repaint;
end;

procedure TForm1.Button1Click(Sender: TObject);
var x,y:integer;
begin
ViewBmp.PixelFormat:=pf24bit;

For y:=0 to ViewBmp.Height-1 do
   For x:=0 to ViewBmp.Width-1 do
     PRGBTriple(ViewBmp.ScanLine[Y])[X].rgbtBlue:=0; //Einfach mal Blau auf 0 setzen fuer das ganze Bitmap
     
  AssignOpBitmapToBitmap(ViewBmp, Image1.Picture.Bitmap);
  Combobox1.ItemIndex := Ord(ViewBmp.PixelFormat) - 1;
  Image1.Repaint;
end;

procedure TForm1.Button3Click(Sender: TObject);
var AlphaBmp: TCanvasOPBitmap;
begin
  AlphaBmp := TCanvasOPBitmap.Create;
  AlphaBmp.Width := 300;
  AlphaBmp.Height := 300;
  AlphaBmp.Canvas.Brush.color := clyellow;
  AlphaBmp.Canvas.FillRect(Rect(0, 0, 300, 300));
  AlphaBmp.SetAlpha(50);
  ViewBmp.AlphaBlend := true;
  ViewBmp.Canvas.Draw(100, 100, AlphaBmp);
  ViewBmp.AlphaBlend := false;
  AssignOpBitmapToBitmap(ViewBmp, Image1.Picture.Bitmap);
  AlphaBmp.free;
  Image1.Repaint;
end;

procedure TForm1.ComboBox2CloseUp(Sender: TObject);
begin
  if ViewBmp.BPP <= 8 then
    ComboBox1CloseUp(self);
end;

procedure TForm1.FormChangeBounds(Sender: TObject);
begin
  if Assigned(ViewBmp) then
  begin
    ViewBmp.Assign(OrigBmp);
    ViewBmp.Canvas.Resample(Image1.width, Image1.height);
    AssignOpBitmapToBitmap(ViewBmp, Image1.Picture.Bitmap);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  OrigBmp := TCanvasOPBitmap.create;
  ViewBmp := TCanvasOPBitmap.create;
  OrigBmp.LoadFromFile(ExtractFilePath(Paramstr(0)) + 'halle.opb');
  ViewBmp.Assign(OrigBmp);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ViewBmp.free;
  OrigBmp.free;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  ViewBmp.Canvas.Resample(Image1.width, Image1.height);
  AssignOpBitmapToBitmap(ViewBmp, Image1.Picture.Bitmap);

  Combobox1.ItemIndex := Ord(ViewBmp.PixelFormat) - 1;
end;

procedure TForm1.chbFromOrigChange(Sender: TObject);
begin
  ComboBox1CloseUp(self);
end;

procedure TForm1.PaintPalette;
var Rec: TRect;
  x, y, i: integer;
const boxsize = 8;
begin
  Image2.Canvas.Brush.Color := clGray;
  Image2.Canvas.FillRect(Rect(0, 0, Image2.width, Image2.Height));

  if ViewBmp.BPP <= 8 then
  begin
    y := boxsize;
    x := boxsize;
    for i := 0 to ViewBmp.ColorTableSize - 1 do
    begin
      if i mod 16 = 0 then
      begin
        inc(y, boxsize + 1);
        x := boxsize;
      end else inc(x, boxsize + 1);
      Rec.Left := x;
      Rec.Top := y;
      Rec.Right := x + boxsize;
      Rec.Bottom := y + boxsize;
      Image2.Canvas.Brush.Color := ViewBmp.ColorTable^[i];
      Image2.Canvas.FillRect(Rec);
    end;
  end;
end;

initialization
{$I demo.lrs}

end.
